#!/usr/bin/env python

from localization_utils import *
import argparse


def parse_args():
    """ Parses the arguments given in the command line

    Returns:
        args: The configured arguments will be attributes of the returned object.
    """
    parser = argparse.ArgumentParser(description='Merging the old translations and the new ones.')

    parser.add_argument("updated_localizable_file", help="The updated localization strings file (produced by genstrings.sh).")

    parser.add_argument("old_translated_file", help="The old translated strings file (with all previously translated strings).")

    parser.add_argument("new_translated_file", help="The new translated strings file (with the new translations).")

    parser.add_argument("merged_translated_file", help="The output file with the merged translations.")

    parser.add_argument("--log_path", default="", help="The log file path")

    return parser.parse_args()


def localization_merge_back(updated_localizable_file, old_translated_file, new_translated_file, merged_translated_file):
    """ Generates a file merging the old translations and the new ones.

    Args:
        updated_localizable_file (str): The path to the updated localization strings file, meaning the strings that
            require translation.
        old_translated_file (str): The path to the strings file containing the previously translated strings.
        new_translated_file (str): The path to the strings file containing the newly translated strings.
        merged_translated_file (str): The path to the output file with the merged translations.

    """
    output_file_elements = []
    old_translated_file_dict = generate_localization_key_to_entry_dictionary_from_file(old_translated_file)
    new_translated_file_dict = generate_localization_key_to_entry_dictionary_from_file(new_translated_file)

    f = open_strings_file(updated_localizable_file, "r")

    for header_comment, comments, key, value in extract_header_comment_key_value_tuples_from_file(f):
        translation_value = None
        if len(header_comment) > 0:
            output_file_elements.append(Comment(header_comment))

        if value in new_translated_file_dict:
            translation_value = new_translated_file_dict[value].value
        elif key in old_translated_file_dict:
            translation_value = old_translated_file_dict[key].value
        elif key in new_translated_file_dict:
            translation_value = new_translated_file_dict[key].value

        if translation_value is not None:
            output_file_elements.append(LocalizationEntry(comments, key, translation_value))

    f.close()

    write_file_elements_to_strings_file(merged_translated_file, output_file_elements)


# The main method for simple command line run.
if __name__ == "__main__":
    args = parse_args()
    setup_logging(args)

    localization_merge_back(args.updated_localizable_file, args.old_translated_file, args.new_translated_file,
                            args.merged_translated_file)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse

from core.localization_utils import *
from jtlocalize.core.localization_commandline_operation import LocalizationCommandLineOperation


class Replacement:
    def __init__(self):
        pass

    def replace_value(self, value):
        pass


class StaticReplacement(Replacement):
    def __init__(self, static_val):
        Replacement.__init__(self)
        self.static_val = static_val.decode("utf8")

    def replace_value(self, _value):
        return self.static_val


class WrapReplacement(Replacement):
    def __init__(self, wrapping_prefix):
        Replacement.__init__(self)
        self.wrapping_prefix = wrapping_prefix.decode("utf8")

    def replace_value(self, value):
        return u"%s(%s)" % (self.wrapping_prefix, value)


def replace_english_values(filename, replacement):
    f = open_strings_file(filename, "r+")
    localization_entries = []
    for header_comment, comments, key, value in extract_header_comment_key_value_tuples_from_file(f):
        localization_entries.append(LocalizationEntry(comments, key, replacement.replace_value(key)))
    f.close()

    write_file_elements_to_strings_file(filename, localization_entries)


PRESET_REPLACEMENTS = {"chicken": StaticReplacement("Chicken"),
                       "chinese": StaticReplacement("鸡鸡鸡"),
                       "localized": WrapReplacement("LOCALIZED")}


class MockTranslateOperation(LocalizationCommandLineOperation):

    def name(self):
        return "mock_translate"

    def description(self):
        return "Localizes a given file to the given string values."

    def configure_parser(self, parser):
        parser.add_argument("file_to_mock", help="The file to localize.")

        localization_option_group = parser.add_mutually_exclusive_group(required=True)

        localization_option_group.add_argument("--preset", choices=PRESET_REPLACEMENTS.keys(),
                                               help="Choose a preset type of mock translation.")

        localization_option_group.add_argument("--static", help="Choose a static replacement of your own.")

        localization_option_group.add_argument("--wrap", help="Choose a wrapping text of your own "
                                                              "(will wrap the original string with parenthesis).")

    def run(self, parsed_args):
        mock_translate(parsed_args.file_to_mock, parsed_args.preset, parsed_args.static, parsed_args.wrap)


def mock_translate(file_to_mock, preset=None, static=None, wrap=None):
    if preset:
        replace_english_values(file_to_mock, PRESET_REPLACEMENTS[preset])

    elif static:
        replace_english_values(file_to_mock, StaticReplacement(static))

    elif wrap:
        replace_english_values(file_to_mock, WrapReplacement(wrap))


if __name__ == "__main__":
    operation = MockTranslateOperation()
    operation.run_with_standalone_parser()




#!/usr/bin/env python

import glob

from jtlocalize.core.localization_merge_back import localization_merge_back
from jtlocalize.core.localization_utils import *
from jtlocalize.configuration.localization_configuration import *
from jtlocalize.core.localization_commandline_operation import LocalizationCommandLineOperation


TRANSLATED_SUFFIX = ".translated"


class MergeTranslationsOperation(LocalizationCommandLineOperation):

    def name(self):
        return "merge"

    def description(self):
        return "Merge translated files to the localization bundle."

    def configure_parser(self, parser):
        super(MergeTranslationsOperation, self).configure_parser(parser)
        parser.add_argument("localization_bundle_path", default=LOCALIZATION_BUNDLE_PATH,
                            help="The path to the localizable bundle.")

    def run(self, parsed_args):
        setup_logging(parsed_args)

        merge_translations(parsed_args.localization_bundle_path)


def merge_translations(localization_bundle_path):
    """ Merges the new translation with the old one.

    The translated files are saved as '.translated' file, and are merged with old translated file.

    Args:
        localization_bundle_path (str): The path to the localization bundle.

    """
    logging.info("Merging translations")
    for lang_dir in os.listdir(localization_bundle_path):
        if lang_dir == DEFAULT_LANGUAGE_DIRECTORY_NAME:
            continue
        for translated_path in glob.glob(os.path.join(localization_bundle_path, lang_dir, "*" + TRANSLATED_SUFFIX)):
            strings_path = translated_path[:-1 * len(TRANSLATED_SUFFIX)]
            localizable_path = os.path.join(localization_bundle_path,
                                            DEFAULT_LANGUAGE_DIRECTORY_NAME,
                                            os.path.basename(strings_path))

            localization_merge_back(localizable_path, strings_path, translated_path, strings_path)


# The main method for simple command line run.
if __name__ == "__main__":

    operation = MergeTranslationsOperation()
    operation.run_with_standalone_parser()



from abc import abstractmethod, ABCMeta
import argparse


class LocalizationCommandLineOperation(object):

    __metaclass__ = ABCMeta

    @abstractmethod
    def name(self):
        """
        Return:
            str: The name of the operation, as will be used as an argument to the master script.
        """
        pass

    @abstractmethod
    def description(self):
        """
        Return:
            str: The one-liner description of the operation, as will be used as the help for it in the usage.
        """
        pass

    def configure_parser(self, parser):
        """
        Adds the necessary supported arguments to the argument parser.

        Args:
            parser (argparse.ArgumentParser): The parser to add arguments to.
        """
        parser.add_argument("--log_path", default="", help="The log file path")
        parser.add_argument("--verbose", help="Increase logging verbosity", action="store_true")

    @abstractmethod
    def run(self, parsed_args):
        """
        Will run the localization operation.

        Args:
            parsed_args: The result of parser.parse_args() after configured by the configure_parser method.
        """
        pass

    def run_with_standalone_parser(self):
        """
        Will run the operation as standalone with a new ArgumentParser
        """
        parser = argparse.ArgumentParser(description=self.description())
        self.configure_parser(parser)
        self.run(parser.parse_args())






#!/usr/bin/env python

from localization_utils import *
import argparse

VALUE_PLACEHOLDER = "XXX"


def parse_args():
    """ Parses the arguments given in the command line

    Returns:
        args: The configured arguments will be attributes of the returned object.
    """
    parser = argparse.ArgumentParser(
        description='Perform a diff on the new localizable file, and the file we already translated.')

    parser.add_argument("input_localizable_file", help="The file that requires translation.")

    parser.add_argument("input_translated_file", help="The file that was already translated.")

    parser.add_argument("excluded_strings_file",
                        help="A file that contains all strings we want to exclude from diff.")

    parser.add_argument("output_translation_file", help="Output file to send for translation.")

    parser.add_argument("--log_path", default="", help="The log file path")

    return parser.parse_args()


def localization_diff(localizable_file, translated_file, excluded_strings_file, output_translation_file):
    """ Generates a strings file representing the strings that were yet to be translated.

    Args:
        localizable_file (str): The path to the localization strings file, meaning the file that represents the strings
            that require translation.
        translated_file (str): The path to the translated strings file, meaning the file containing the strings that
            were already translated.
        excluded_strings_file (str): The path to a file that contains all the strings we want to exclude from this and
            from future diffs.
        output_translation_file (str): The path to the output file, which will contain the strings the require
            translation, but are not in the already given translation file.
    """
    old_translated_file_dictionary = generate_localization_key_to_entry_dictionary_from_file(translated_file)
    if excluded_strings_file is not None and os.path.isfile(excluded_strings_file):
        excluded_file_dictionary = generate_localization_key_to_entry_dictionary_from_file(excluded_strings_file)
    else:
        excluded_file_dictionary = {}

    # The reason we keep a list of the keys, and not just pop is because values can repeat themselves.
    translated_list = old_translated_file_dictionary.keys()
    output_dictionary = {}
    output_file_elements = []
    f = open_strings_file(localizable_file, "r")

    output_file_elements.append(Comment(u"""
/**
 * This file contains all the strings that were extracted from our app and that need to be translated.
 * Each entry may or may not have a comment explaining context, and a "key" = "%s" equation.
 * To localize, you need to fill the right side of the equation with the translation of the left side.
 * Please keep special expressions such as '%%@' or '%%1$@' as is. Usually the comment will explain their context.
 */
""" % (VALUE_PLACEHOLDER,)))

    for _header_comment, comments, key, value in extract_header_comment_key_value_tuples_from_file(f):
        if key in translated_list or key in excluded_file_dictionary:
            if key in old_translated_file_dictionary:
                old_translated_file_dictionary.pop(key)
        elif value in output_dictionary:
            output_dictionary[value].add_comments(comments)
            output_file_elements.append(Comment(
                u"/* There was a value '%s' here but it was a duplicate of an older value and removed. */\n" % value))
        else:
            loc_obj = LocalizationEntry(comments, value, VALUE_PLACEHOLDER)
            output_dictionary[value] = loc_obj
            output_file_elements.append(loc_obj)

    for key, removed_trans in old_translated_file_dictionary.items():
        output_file_elements.append(Comment(u"""
/*
 * Entry removed from previous translation file:
 * %s
 * "%s" = "%s";
 */
""" % (", ".join(removed_trans.comments), removed_trans.key, removed_trans.value)))

    write_file_elements_to_strings_file(output_translation_file, output_file_elements)

# The main method for simple command line run.
if __name__ == "__main__":
    args = parse_args()
    setup_logging(args)

    localization_diff(args.input_localizable_file, args.input_translated_file, args.excluded_strings_file,
                      args.output_translation_file)



#!/usr/bin/env python
import shutil
import subprocess
import sys

from jtlocalize.core.localization_utils import *
from jtlocalize.configuration.localization_configuration import *
from jtlocalize.core.add_genstrings_comments_to_file import add_genstrings_comments_to_file
from jtlocalize.core.create_localized_strings_from_ib_files import create_localized_strings_from_ib_files
from jtlocalize.core.handle_duplicates_in_localization import handle_duplications
from jtlocalize.core.merge_strings_files import merge_strings_files
from jtlocalize.core.localization_commandline_operation import LocalizationCommandLineOperation


LOCALIZATION_FILENAME = "Localizable.strings"
DEFAULT_TMP_DIRECTORY = "/tmp"
NO_COMMENT_PROVIDED_STRING = "No comment provided by engineer."


class GenerateStringsFileOperation(LocalizationCommandLineOperation):
    def name(self):
        return "generate"

    def description(self):
        return "Create the unified strings file."

    def configure_parser(self, parser):
        super(GenerateStringsFileOperation, self).configure_parser(parser)

        parser.add_argument("project_base_directory", help="The directory in which the app code lays")

        parser.add_argument("localization_bundle_path", default=LOCALIZATION_BUNDLE_PATH,
                            help="The path to the localizable bundle.")

        parser.add_argument("--tmp_directory", default=DEFAULT_TMP_DIRECTORY,
                            help="The default temporary directory to write files to in the process")

        parser.add_argument("--special_ui_components_prefix",
                            help="By default script will warn about ui components using JTL comments that aren't using "
                                 "the JT prefix (e.g. JTLabel). if you pass a value like XYZ here, it won't warn about "
                                 "components like XYZButton, XYZLabel, etc.")

        parser.add_argument("--exclude_dirs", nargs='+',
                            help="Directories to exclude when looking for source files to extract strings from")

        parser.add_argument("--include_strings_from_file",
                            help="Option to add additional strings from a file other the ones extracted with genstrings")

    def run(self, parsed_args):
        setup_logging(parsed_args)

        generate_strings(parsed_args.project_base_directory,
                         parsed_args.localization_bundle_path,
                         parsed_args.tmp_directory,
                         parsed_args.exclude_dirs,
                         parsed_args.include_strings_from_file,
                         parsed_args.special_ui_components_prefix)


def extract_source_files(base_dir, exclude_dirs):
    return find_files(base_dir, [".m", ".mm", ".swift"], exclude_dirs)


def generate_strings(project_base_dir, localization_bundle_path, tmp_directory, exclude_dirs, include_strings_file,
                     special_ui_components_prefix):
    """
    Calls the builtin 'genstrings' command with JTLocalizedString as the string to search for,
    and adds strings extracted from UI elements internationalized with 'JTL' + removes duplications.
    """

    localization_directory = os.path.join(localization_bundle_path, DEFAULT_LANGUAGE_DIRECTORY_NAME)
    if not os.path.exists(localization_directory):
        os.makedirs(localization_directory)

    localization_file = os.path.join(localization_directory, LOCALIZATION_FILENAME)

    # Creating the same directory tree structure in the tmp directory
    tmp_localization_directory = os.path.join(tmp_directory, DEFAULT_LANGUAGE_DIRECTORY_NAME)
    tmp_localization_file = os.path.join(tmp_localization_directory, LOCALIZATION_FILENAME)

    if os.path.isdir(tmp_localization_directory):
        shutil.rmtree(tmp_localization_directory)
    os.mkdir(tmp_localization_directory)

    logging.info("Running genstrings")

    source_files = extract_source_files(project_base_dir, exclude_dirs)

    genstrings_cmd = 'genstrings -s JTLocalizedString -o %s %s' % (tmp_localization_directory, " ".join(
            ['"%s"' % (source_file,) for source_file in source_files]))

    genstrings_process = subprocess.Popen(genstrings_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                          stdin=subprocess.PIPE, shell=True)

    genstrings_out, genstrings_err = genstrings_process.communicate()

    remove_empty_comments_from_file(tmp_localization_file)
    add_genstrings_comments_to_file(tmp_localization_file, genstrings_err)

    genstrings_rc = genstrings_process.returncode
    if genstrings_rc != 0:
        logging.fatal("genstrings returned %d, aborting run!", genstrings_rc)
        sys.exit(genstrings_rc)

    create_localized_strings_from_ib_files(project_base_dir, exclude_dirs, tmp_localization_file,
                                           special_ui_components_prefix)

    if include_strings_file:
        target = open_strings_file(tmp_localization_file, "a")
        source = open_strings_file(include_strings_file, "r")
        target.write(source.read())
        source.close()
        target.close()

    handle_duplications(tmp_localization_file)

    if os.path.isfile(localization_file):
        logging.info("Merging old localizable with new one...")
        merge_strings_files(localization_file, tmp_localization_file)
    else:
        logging.info("No Localizable yet, moving the created file...")
        shutil.move(tmp_localization_file, localization_file)


def remove_empty_comments_from_file(file_path):
    if not os.path.exists(file_path):
        return

    orig_file = open_strings_file(file_path, "r")
    filtered_path = file_path + ".filtered"
    filtered_file = open_strings_file(filtered_path, "w")
    for line in orig_file.readlines():
        if NO_COMMENT_PROVIDED_STRING in line:
            processed_line = u"/* */\n"
        else:
            processed_line = line

        filtered_file.write(processed_line)

    orig_file.close()
    filtered_file.close()
    os.remove(file_path)
    shutil.move(filtered_path, file_path)


# The main method for simple command line run.
if __name__ == "__main__":
    operation = GenerateStringsFileOperation()
    operation.run_with_standalone_parser()



#!/usr/bin/env python

import argparse
from xml.dom import minidom
from xml.sax.saxutils import unescape

from localization_utils import *

# The prefix to identify a comment for an internationalized comment.
JT_INTERNATIONALIZED_COMMENT_PREFIX = 'jtl_'

DEFAULT_UI_COMPONENTS_PREFIX = 'JT'


def write_string_pairs_from_ib_file_to_file(ib_files_directory, exclude_dirs, output_file,
                                            special_ui_components_prefix):
    logging.info('Creating localization string pairs from IB files')

    string_pairs = extract_string_pairs_in_dir(ib_files_directory, exclude_dirs, special_ui_components_prefix)
    output_file_desc = open_strings_file(output_file, "a")
    write_section_header_to_file(output_file_desc, 'IB Files Section')
    for entry_key, entry_comment in string_pairs:
        output_file_desc.write('\n')
        if entry_key is not None:
            write_entry_to_file(output_file_desc, entry_comment, entry_key)
        else:
            write_section_header_to_file(output_file_desc, entry_comment)

    output_file_desc.close()


def extract_string_pairs_in_dir(directory, exclude_dirs, special_ui_components_prefix):
    """ Extract string pairs in the given directory's xib/storyboard files.

    Args:
        directory (str): The path to the directory.
        exclude_dirs (str): A list of directories to exclude from extraction.
        special_ui_components_prefix (str):
            If not None, extraction will not warn about internationalized UI components with this class prefix.

    Returns:
        list: The extracted string pairs for all IB files in the directory.

    """
    result = []
    for ib_file_path in find_files(directory, [".xib", ".storyboard"], exclude_dirs):
        result += extract_string_pairs_in_ib_file(ib_file_path, special_ui_components_prefix)

    return result


def get_element_attribute_or_empty(element, attribute_name):
    """

    Args:
        element (element): The xib's element.
        attribute_name (str): The desired attribute's name.

    Returns:
        The attribute's value, or an empty str if none exists.

    """
    return element.attributes[attribute_name].value if element.hasAttribute(attribute_name) else ""


def extract_element_internationalized_comment(element):
    """ Extracts the xib element's comment, if the element has been internationalized.

    Args:
        element (element): The element from which to extract the comment.

    Returns:
        The element's internationalized comment, None if it does not exist, or hasn't been internationalized (according
        to the JTLocalize definitions).

    """
    element_entry_comment = get_element_attribute_or_empty(element, 'userLabel')
    if element_entry_comment == "":
        try:
            element_entry_comment = element.getElementsByTagName('string')[0].firstChild.nodeValue
        except Exception:
            element_entry_comment = ""
    if not element_entry_comment.lower().startswith(JT_INTERNATIONALIZED_COMMENT_PREFIX):
        return None
    else:
        return element_entry_comment[len(JT_INTERNATIONALIZED_COMMENT_PREFIX):]


def warn_if_element_not_of_class(element, class_suffix, special_ui_components_prefix):
    """ Log a warning if the element is not of the given type (indicating that it is not internationalized).

    Args:
        element: The xib's XML element.
        class_name: The type the element should be, but is missing.
        special_ui_components_prefix: If provided, will not warn about class with this prefix (default is only 'JT')
    """
    valid_class_names = ["%s%s" % (DEFAULT_UI_COMPONENTS_PREFIX, class_suffix)]
    if special_ui_components_prefix is not None:
        valid_class_names.append("%s%s" % (special_ui_components_prefix, class_suffix))

    if (not element.hasAttribute('customClass')) or element.attributes['customClass'].value not in valid_class_names:
        logging.warn("WARNING: %s is internationalized but isn't one of %s",
                     extract_element_internationalized_comment(element), valid_class_names)


def add_string_pairs_from_attributed_ui_element(results, ui_element, comment_prefix):
    """ Adds string pairs from a UI element with attributed text

    Args:
        results (list): The list to add the results to.
        attributed_element (element): The element from the xib that contains, to extract the fragments from.
        comment_prefix (str): The prefix of the comment to use for extracted string
                              (will be appended "Part X" suffices)

    Returns:
        bool: Whether or not an attributed string was found.
    """
    attributed_strings = ui_element.getElementsByTagName('attributedString')
    if attributed_strings.length == 0:
        return False

    attributed_element = attributed_strings[0]
    fragment_index = 1
    for fragment in attributed_element.getElementsByTagName('fragment'):
        # The fragment text is either as an attribute <fragment content="TEXT">
        # or a child in the format <string key='content'>TEXT</string>
        try:
            label_entry_key = fragment.attributes['content'].value
        except KeyError:
            label_entry_key = fragment.getElementsByTagName('string')[0].firstChild.nodeValue

        comment = "%s Part %d" % (comment_prefix, fragment_index)
        results.append((label_entry_key, comment))
        fragment_index += 1

    return fragment_index > 1


def add_string_pairs_from_label_element(xib_file, results, label, special_ui_components_prefix):
    """ Adds string pairs from a label element.

    Args:
        xib_file (str): Path to the xib file.
        results (list): The list to add the results to.
        label (element): The label element from the xib, to extract the string pairs from.
        special_ui_components_prefix (str):
            If not None, extraction will not warn about internationalized UI components with this class prefix.

    """
    label_entry_comment = extract_element_internationalized_comment(label)
    if label_entry_comment is None:
        return

    warn_if_element_not_of_class(label, 'Label', special_ui_components_prefix)

    if label.hasAttribute('usesAttributedText') and label.attributes['usesAttributedText'].value == 'YES':
        add_string_pairs_from_attributed_ui_element(results, label, label_entry_comment)
    else:
        try:
            label_entry_key = label.attributes['text'].value
        except KeyError:
            try:
                label_entry_key = label.getElementsByTagName('string')[0].firstChild.nodeValue
            except Exception:
                label_entry_key = 'N/A'
                logging.warn("%s: Missing text entry in %s", xib_file, label.toxml('UTF8'))
        results.append((label_entry_key, label_entry_comment))


def add_string_pairs_from_text_field_element(xib_file, results, text_field, special_ui_components_prefix):
    """ Adds string pairs from a textfield element.

    Args:
        xib_file (str): Path to the xib file.
        results (list): The list to add the results to.
        text_field(element): The textfield element from the xib, to extract the string pairs from.
        special_ui_components_prefix (str):
            If not None, extraction will not warn about internationalized UI components with this class prefix.

    """
    text_field_entry_comment = extract_element_internationalized_comment(text_field)
    if text_field_entry_comment is None:
        return

    if text_field.hasAttribute('usesAttributedText') and text_field.attributes['usesAttributedText'].value == 'YES':
        add_string_pairs_from_attributed_ui_element(results, text_field, text_field_entry_comment)
    else:
        try:
            text_field_entry_key = text_field.attributes['text'].value
            results.append((text_field_entry_key, text_field_entry_comment + ' default text value'))
        except KeyError:
            pass
    try:
        text_field_entry_key = text_field.attributes['placeholder'].value
        results.append((text_field_entry_key, text_field_entry_comment + ' placeholder text value'))
    except KeyError:
        pass
    warn_if_element_not_of_class(text_field, 'TextField', special_ui_components_prefix)


def add_string_pairs_from_text_view_element(xib_file, results, text_view, special_ui_components_prefix):
    """ Adds string pairs from a textview element.

    Args:
        xib_file (str): Path to the xib file.
        results (list): The list to add the results to.
        text_view(element): The textview element from the xib, to extract the string pairs from.
        special_ui_components_prefix(str): A custom prefix for internationalize component to allow (default is only JT)

    """
    text_view_entry_comment = extract_element_internationalized_comment(text_view)
    if text_view_entry_comment is None:
        return

    if text_view.hasAttribute('usesAttributedText') and text_view.attributes['usesAttributedText'].value == 'YES':
        add_string_pairs_from_attributed_ui_element(results, text_view, text_view_entry_comment)
    else:
        try:
            text_view_entry_key = text_view.attributes['text'].value
            results.append((text_view_entry_key, text_view_entry_comment + ' default text value'))
        except KeyError:
            pass
    warn_if_element_not_of_class(text_view, 'TextView', special_ui_components_prefix)


def add_string_pairs_from_button_element(xib_file, results, button, special_ui_components_prefix):
    """ Adds strings pairs from a button xib element.

    Args:
        xib_file (str): Path to the xib file.
        results (list): The list to add the results to.
        button(element): The button element from the xib, to extract the string pairs from.
        special_ui_components_prefix(str): A custom prefix for internationalize component to allow (default is only JT)

    """
    button_entry_comment = extract_element_internationalized_comment(button)
    if button_entry_comment is None:
        return

    for state in button.getElementsByTagName('state'):
        state_name = state.attributes['key'].value
        state_entry_comment = button_entry_comment + " - " + state_name + " state of button"
        if not add_string_pairs_from_attributed_ui_element(results, state, state_entry_comment):
            try:
                button_entry_key = state.attributes['title'].value
            except KeyError:
                try:
                    button_entry_key = state.getElementsByTagName('string')[0].firstChild.nodeValue
                except Exception:
                    continue

            results.append((button_entry_key, state_entry_comment))

    warn_if_element_not_of_class(button, 'Button', special_ui_components_prefix)


def extract_string_pairs_in_ib_file(file_path, special_ui_components_prefix):
    """ Extract the strings pairs (key and comment) from a xib file.

    Args:
        file_path (str): The path to the xib file.
        special_ui_components_prefix (str):
            If not None, extraction will not warn about internationalized UI components with this class prefix.

    Returns:
        list: List of tuples representing the string pairs.

    """
    try:
        results = []
        xmldoc = minidom.parse(file_path)

        element_name_to_add_func = {'label': add_string_pairs_from_label_element,
                                    'button': add_string_pairs_from_button_element,
                                    'textField': add_string_pairs_from_text_field_element,
                                    'textView': add_string_pairs_from_text_view_element}

        for element_name in element_name_to_add_func:
            add_func = element_name_to_add_func[element_name]
            elements = xmldoc.getElementsByTagName(element_name)
            for element in elements:
                add_func(file_path, results, element, special_ui_components_prefix)

        # Find strings of format JTL('Key Name', 'Key Comment') and add them to the results
        jtl_brackets_find_results = re.findall(JTL_REGEX, open(file_path).read())
        unescaped_jtl_brackets_find_results = [(unescape(x), unescape(y)) for (x, y) in jtl_brackets_find_results]
        results += unescaped_jtl_brackets_find_results

        if len(results) > 0:
            results = [(None, os.path.basename(file_path))] + results
        return results

    except Exception, e:
        logging.warn("ERROR: Error processing %s (%s: %s)", file_path, type(e), str(e))
        return []


def create_localized_strings_from_ib_files(ib_files_directory, exclude_dirs, output_file,
                                           special_ui_components_prefix=None):
    write_string_pairs_from_ib_file_to_file(ib_files_directory, exclude_dirs, output_file, special_ui_components_prefix)


def parse_args():
    """ Parses the arguments given in the command line

    Returns:
        args: The configured arguments will be attributes of the returned object.
    """
    parser = argparse.ArgumentParser(description='Extract the string for localization from IB files directory.')

    parser.add_argument("ib_files_directory", help="The directory containing the IB files.")

    parser.add_argument("output_file", help="The output file.")

    parser.add_argument("--special_ui_components_prefix", 
                        help="By default script will warn about ui components using JTL comments that aren't using the "
                             "JT prefix (e.g. JTLabel). if you pass a value like XYZ here, it won't warn about "
                             "components like XYZButton, XYZLabel, etc.")

    parser.add_argument("--exclude_dirs", nargs='+',
                        help="Directories to exclude when looking for IB files to extract strings from")

    parser.add_argument("--log_path", default="", help="The log file path")

    return parser.parse_args()

# The main method for simple command line run.
if __name__ == '__main__':

    args = parse_args()
    setup_logging(args)

    create_localized_strings_from_ib_files(args.ib_files_directory, args.exclude_dirs, args.output_file,
                                           args.special_ui_components_prefix)





#!/usr/bin/env python

from localization_utils import *
import argparse


def parse_args():
    """ Parses the arguments given in the command line

    Returns:
        args: The configured arguments will be attributes of the returned object.
    """
    parser = argparse.ArgumentParser(description='Omits duplications in a given localizable file.')

    parser.add_argument("localizable_file", help="The file that requires duplication handling.")

    parser.add_argument("--log_path", default="", help="The log file path")

    return parser.parse_args()


def handle_duplications(file_path):
    """ Omits the duplications in the strings files.
        Keys that appear more than once, will be joined to one appearance and the omit will be documented.

    Args:
        file_path (str): The path to the strings file.

    """
    logging.info('Handling duplications for "%s"', file_path)
    f = open_strings_file(file_path, "r+")
    header_comment_key_value_tuples = extract_header_comment_key_value_tuples_from_file(f)
    file_elements = []
    section_file_elements = []
    keys_to_objects = {}
    duplicates_found = []
    for header_comment, comments, key, value in header_comment_key_value_tuples:
        if len(header_comment) > 0:
            # New section - Appending the last section entries, sorted by comment
            for elem in sorted(section_file_elements, key=lambda x: x.comments[0]):
                file_elements.append(elem)
            section_file_elements = []
            file_elements.append(Comment(header_comment))

        if key in keys_to_objects:
            keys_to_objects[key].add_comments(comments)
            duplicates_found.append(key)
        else:
            loc_obj = LocalizationEntry(comments, key, value)
            keys_to_objects[key] = loc_obj
            section_file_elements.append(loc_obj)

    # Adding last section
    for elem in sorted(section_file_elements, key=lambda x: x.comments[0]):
        file_elements.append(elem)

    f.seek(0)

    for element in file_elements:
        f.write(unicode(element))
        f.write(u"\n")

    f.truncate()
    f.close()

    logging.info("Omitted %d duplicates (%s)" % (len(duplicates_found), ",".join(duplicates_found)))
    logging.info('Finished handling duplications for "%s"', file_path)


# The main method for simple command line run.
if __name__ == "__main__":

    args = parse_args()
    setup_logging(args)
    handle_duplications(args.localizable_file)



#!/usr/bin/env python

from localization_utils import *
import sys
import argparse
import re


def parse_args():
    """ Parses the arguments given in the command line

    Returns:
        args: The configured arguments will be attributes of the returned object.
    """
    parser = argparse.ArgumentParser(description='Add the comments of duplicate keys to the strings file.')

    parser.add_argument("localization_file", help="The strings file from genstrings output.")

    parser.add_argument("genstrings_err", help="The stderr output of the genrstrings script.")

    parser.add_argument("--log_path", default="", help="The log file path")

    return parser.parse_args()


def add_genstrings_comments_to_file(localization_file, genstrings_err):
    """ Adds the comments produced by the genstrings script for duplicate keys.

    Args:
        localization_file (str): The path to the strings file.

    """

    errors_to_log = [line for line in genstrings_err.splitlines() if "used with multiple comments" not in line]

    if len(errors_to_log) > 0:
        logging.warning("genstrings warnings:\n%s", "\n".join(errors_to_log))

    loc_file = open_strings_file(localization_file, "a")

    regex_matches = re.findall(r'Warning: Key "(.*?)" used with multiple comments ("[^"]*" (& "[^"]*")+)',
                               genstrings_err)

    logging.info("Adding multiple comments from genstrings output")
    for regex_match in regex_matches:
        if len(regex_match) == 3:
            key = regex_match[0]
            comments = [comment.strip()[1:-1] for comment in regex_match[1].split("&")]

            logging.info("Found key with %d comments: %s", len(comments), key)

            loc_key = LocalizationEntry(comments, key, key)

            loc_file.write(unicode(loc_key))
            loc_file.write(u"\n")

    loc_file.close()


# The main method for simple command line run.
if __name__ == "__main__":

    args = parse_args()
    setup_logging(args)

    add_genstrings_comments_to_file(args.localization_file, args.genstrings_err)


#!/usr/bin/env python

import argparse
from localization_utils import *


def parse_args():
    """ Parses the arguments given in the command line

    Returns:
        args: The configured arguments will be attributes of the returned object.
    """
    parser = argparse.ArgumentParser(description='Merge old strings file with new one.')

    parser.add_argument("old_strings_file", help="The old strings file (the values might have changed).")

    parser.add_argument("new_strings_file", help="The new strings file (produced by genstrings.sh.")

    parser.add_argument("--log_path", default="", help="The log file path")

    return parser.parse_args()


def merge_strings_files(old_strings_file, new_strings_file):
    """ Merges the old strings file with the new one.

    Args:
        old_strings_file (str): The path to the old strings file (previously produced, and possibly altered)
        new_strings_file (str): The path to the new strings file (newly produced).

    """
    old_localizable_dict = generate_localization_key_to_entry_dictionary_from_file(old_strings_file)
    output_file_elements = []

    f = open_strings_file(new_strings_file, "r+")

    for header_comment, comments, key, value in extract_header_comment_key_value_tuples_from_file(f):
        if len(header_comment) > 0:
            output_file_elements.append(Comment(header_comment))

        localize_value = value
        if key in old_localizable_dict:
            localize_value = old_localizable_dict[key].value

        output_file_elements.append(LocalizationEntry(comments, key, localize_value))

    f.close()

    write_file_elements_to_strings_file(old_strings_file, output_file_elements)


# The main method for simple command line run.
if __name__ == "__main__":

    args = parse_args()
    setup_logging(args)

    merge_strings_files(args.old_strings_file, args.new_strings_file)



#!/usr/bin/env python

import codecs
import os
import re
import operator
import logging

from localization_objects import *

# Regexp to parse and inspect strings in format JTL('Key Name', 'Key Comment')
JTL_REGEX = r"""JTL\(\\?['"](.+?)\\?['"],\s*\\?['"](.+?)\\?['"]\)"""

# Regexp to parse and inspect localization entries in the strings file.
HEADER_COMMENT_KEY_VALUE_TUPLES_REGEX = '((/\*\*\* *[^\n]*? *\*\*\*/\n*)*)(/\* *[^;]* *\*/\n*)"(.*?)" *= *"(.*?)";\s*\n'


def rewrite_localization_file_with_entry_modifications(localizable_file, output_file, modification_func):
    file_descriptor = open_strings_file(localizable_file, "r")

    output_file_elements = []

    for header_comment, comments, key, value in extract_header_comment_key_value_tuples_from_file(file_descriptor):

        if len(header_comment) > 0:
            output_file_elements.append(Comment(header_comment))

        after_modification = modification_func(LocalizationEntry(comments, key, value))

        if type(after_modification) == list:
            for entry in after_modification:
                if type(entry) == LocalizationEntry:
                    output_file_elements.append(entry)

        elif type(after_modification) == LocalizationEntry:
            output_file_elements.append(after_modification)

    write_file_elements_to_strings_file(output_file, output_file_elements)


def open_strings_file(file_path, mode):
    """ Open the strings file

    Args:
        file_path (str): The path to the strings file
        mode (str): string representation of the mode in which to open the file (read, write, append, etc.)
    """
    return codecs.open(file_path, mode, "utf-16")


def write_file_elements_to_strings_file(file_path, file_elements):
    """ Write elements to the string file

    Args:
        file_path (str): The path to the strings file
        file_elements (list) : List of elements to write to the file.
    """
    f = open_strings_file(file_path, "w")
    for element in file_elements:
        f.write(unicode(element))
        f.write(u"\n")

    f.close()


def setup_logging(args=None):
    """ Setup logging module.

    Args:
        args (optional): The arguments returned by the argparse module.
    """
    logging_level = logging.WARNING
    if args is not None and args.verbose:
        logging_level = logging.INFO
    config = {"level": logging_level, "format": "jtlocalize:%(message)s"}

    if args is not None and args.log_path != "":
        config["filename"] = args.log_path

    logging.basicConfig(**config)


def __generate_localization_dictionary_from_file(file_path, localization_entry_attribute_name_for_key):
    """ Generates a dictionary mapping between keys (defined by the given attribute name) and localization entries.

    Args:
        file_path (str): The strings file path.
        localization_entry_attribute_name_for_key: The name of the attribute of LocalizationEntry to use as key.

    Returns:
        dict: A dictionary mapping between keys (defined by the given attribute name) and localization entries.
    """
    localization_dictionary = {}
    f = open_strings_file(file_path, "r+")
    header_comment_key_value_tuples = extract_header_comment_key_value_tuples_from_file(f)

    if len(header_comment_key_value_tuples) == 0:
        logging.warning("Couldn't find any strings in file '%s'. Check encoding and format." % file_path)

    for header_comment, comments, key, value in header_comment_key_value_tuples:
        localization_entry = LocalizationEntry(comments, key, value)
        localization_dictionary[
            localization_entry.__getattribute__(localization_entry_attribute_name_for_key)] = localization_entry
    f.close()
    return localization_dictionary


def generate_localization_key_to_entry_dictionary_from_file(file_path):
    """ Generates a dictionary mapping between localization keys and entries.

    Args:
        file_path (str): The strings file path.

    Returns:
        dict: A dictionary mapping between localization keys and entries.
    """
    return __generate_localization_dictionary_from_file(file_path, "key")


def generate_localization_value_to_entry_dictionary_from_file(file_path):
    """ Generates a dictionary mapping between localization values and entries.

    Args:
        file_path (str): The strings file path.

    Returns:
        dict: A dictionary mapping between localization values and entries.
    """
    return __generate_localization_dictionary_from_file(file_path, "value")


def extract_header_comment_key_value_tuples_from_file(file_descriptor):
    """ Extracts tuples representing comments and localization entries from strings file.

    Args:
        file_descriptor (file): The file to read the tuples from

    Returns:
        list : List of tuples representing the headers and localization entries.

    """
    file_data = file_descriptor.read()
    findall_result = re.findall(HEADER_COMMENT_KEY_VALUE_TUPLES_REGEX, file_data, re.MULTILINE | re.DOTALL)

    returned_list = []
    for header_comment, _ignored, raw_comments, key, value in findall_result:
        comments = re.findall("/\* (.*?) \*/", raw_comments)
        if len(comments) == 0:
            comments = [u""]
        returned_list.append((header_comment, comments, key, value))

    return returned_list


def extract_jtl_string_pairs_from_text_file(results_dict, file_path):
    """ Extracts all string pairs matching the JTL pattern from given text file.

    This can be used as an "extract_func" argument in the extract_string_pairs_in_directory method.

    Args:
        results_dict (dict): The dict to add the the string pairs to.
        file_path (str): The path of the file from which to extract the string pairs.

    """
    result_pairs = re.findall(JTL_REGEX, open(file_path).read())
    for result_key, result_comment in result_pairs:
        results_dict[result_key] = result_comment
    return results_dict


def extract_string_pairs_in_directory(directory_path, extract_func, filter_func):
    """ Retrieves all string pairs in the directory

    Args:
        directory_path (str): The path of the directory containing the file to extract string pairs from.
        extract_func (function): Function for extracting the localization keys and comments from the files.
            The extract function receives 2 parameters:
            - dict that the keys (a key in the dict) and comments (a value in the dict) are added to.
            - str representing file path

        filter_func (function): Function for filtering files in the directory.
            The filter function receives the file name and returns a bool representing the filter result.
            True if the file name passed the filter, False otherwise.

    Returns:
        dict: A mapping between string pairs first value (probably the key), and the second value (probably the comment).
    """
    result = {}
    for root, dirnames, filenames in os.walk(directory_path):
        for file_name in filenames:
            if filter_func(file_name):
                file_path = os.path.join(root, file_name)
                try:
                    extract_func(result, file_path)
                except Exception as e:
                    print "Error in file " + file_name
                    print e
    return result


def write_entry_to_file(file_descriptor, entry_comment, entry_key):
    """ Writes a localization entry to the file

    Args:
        file_descriptor (file, instance): The file to write the entry to.
        entry_comment (str): The entry's comment.
        entry_key (str): The entry's key.
    """
    escaped_key = re.sub(r'([^\\])"', '\\1\\"', entry_key)
    file_descriptor.write(u'/* %s */\n' % entry_comment)
    file_descriptor.write(u'"%s" = "%s";\n' % (escaped_key, escaped_key))


def write_section_header_to_file(file_descriptor, section_name):
    """ Writes a section header to the file

    Args:
        file_descriptor (file, instance): The file to writes the section header to.
        section_name (str): The name of the section.
    """
    file_descriptor.write('/*** %s ***/\n' % section_name)


def append_dictionary_to_file(localization_key_to_comment, file_path, section_name):
    """ Appends dictionary of localization keys and comments to a file

    Args:
        localization_key_to_comment (dict): A mapping between localization keys and comments.
        file_path (str): The path of the file to append to.
        section_name (str): The name of the section.

    """
    output_file = open_strings_file(file_path, "a")
    write_section_header_to_file(output_file, section_name)
    for entry_key, entry_comment in sorted(localization_key_to_comment.iteritems(), key=operator.itemgetter(1)):
        output_file.write(u'\n')
        write_entry_to_file(output_file, entry_comment, entry_key)
    output_file.close()


def write_dict_to_new_file(file_name, localization_key_to_comment):
    """ Writes dictionary of localization keys and comments to a file.

    Args:
        localization_key_to_comment (dict): A mapping between localization keys and comments.
        file_name (str): The path of the file to append to.

    """
    output_file_descriptor = open_strings_file(file_name, "w")
    for entry_key, entry_comment in sorted(localization_key_to_comment.iteritems(), key=operator.itemgetter(1)):
        write_entry_to_file(output_file_descriptor, entry_comment, entry_key)
        output_file_descriptor.write(u'\n')
    output_file_descriptor.close()


def find_files(base_dir, extensions, exclude_dirs=list()):
    """ Find all files matching the given extensions.

    Args:
        base_dir (str): Path of base directory to search in.
        extensions (list): A list of file extensions to search for.
        exclude_dirs (list): A list of directories to exclude from search.

    Returns:
        list of paths that match the search
    """
    result = []
    for root, dir_names, file_names in os.walk(base_dir):
        for filename in file_names:
            candidate = os.path.join(root, filename)
            if should_include_file_in_search(candidate, extensions, exclude_dirs):
                result.append(candidate)
    return result


def should_include_file_in_search(file_name, extensions, exclude_dirs):
    """ Whether or not a filename matches a search criteria according to arguments.

    Args:
        file_name (str): A file path to check.
        extensions (list): A list of file extensions file should match.
        exclude_dirs (list): A list of directories to exclude from search.

    Returns:
        A boolean of whether or not file matches search criteria.

    """
    return (exclude_dirs is None or not any(file_name.startswith(d) for d in exclude_dirs)) and \
        any(file_name.endswith(e) for e in extensions)



#!/usr/bin/env python

from distutils.core import setup

setup(name='jtlocalize',
      version='0.1.4',
      description='iOS localization framework',
      author='JoyTunes',
      author_email='info@joytunes.com',
      url='https://github.com/joytunes/JTLocalize/',
      packages=['jtlocalize','jtlocalize.core', 'jtlocalize.configuration'],
      scripts=['jtlocalize/bin/jtlocalize'],
      license='MIT',
     )



#!/usr/bin/env python

import shutil

from jtlocalize.core.localization_commandline_operation import LocalizationCommandLineOperation
from jtlocalize.core.localization_diff import localization_diff
from jtlocalize.core.localization_utils import *
from jtlocalize.configuration.localization_configuration import *


class PrepareForTranslationOperation(LocalizationCommandLineOperation):

    def name(self):
        return "prepare_diff"

    def description(self):
        return "Prepare the localization bundle for translation with only the necessary diff."

    def configure_parser(self, parser):
        super(PrepareForTranslationOperation, self).configure_parser(parser)
        
        parser.add_argument("localization_bundle_path", default=LOCALIZATION_BUNDLE_PATH,
                            help="The path to the localizable bundle.")

    def run(self, parsed_args):
        setup_logging(parsed_args)

        prepare_for_translation(parsed_args.localization_bundle_path)


def prepare_for_translation(localization_bundle_path):
    """ Prepares the localization bundle for translation.

    This means, after creating the strings files using genstrings.sh, this will produce '.pending' files, that contain
    the files that are yet to be translated.

    Args:
        localization_bundle_path (str): The path to the localization bundle.

    """

    logging.info("Preparing for translation..")

    for strings_file in os.listdir(os.path.join(localization_bundle_path, DEFAULT_LANGUAGE_DIRECTORY_NAME)):
        if not strings_file.endswith(".strings"):
            continue

        strings_path = os.path.join(localization_bundle_path, DEFAULT_LANGUAGE_DIRECTORY_NAME, strings_file)
        for lang_dir in os.listdir(localization_bundle_path):
            if lang_dir == DEFAULT_LANGUAGE_DIRECTORY_NAME or lang_dir.startswith("."):
                continue

            dest_strings_path = os.path.join(localization_bundle_path, lang_dir, strings_file)
            pending_path = dest_strings_path + ".pending"
            excluded_path = dest_strings_path + ".excluded"
            if not os.path.exists(dest_strings_path):
                open_strings_file(dest_strings_path, "a").close()

            logging.info("Preparing diff for %s in %s", lang_dir, pending_path)
            localization_diff(strings_path, dest_strings_path, excluded_path, pending_path)



# The main method for simple command line run.
if __name__ == "__main__":

    operation = PrepareForTranslationOperation()
    operation.run_with_standalone_parser()




import argparse
from core.localization_utils import extract_header_comment_key_value_tuples_from_file, open_strings_file
from jtlocalize.core.localization_commandline_operation import LocalizationCommandLineOperation

__author__ = 'lwager'


class WordCountOperation(LocalizationCommandLineOperation):

    def name(self):
        return "word_count"

    def description(self):
        return "Perform a word count on pending localization file."

    def configure_parser(self, parser):
        parser.add_argument("localizable_file", help="The file that requires translation.")

    def run(self, parsed_args):
        print word_count(parsed_args.localizable_file)


def word_count(f_name):

    f = open_strings_file(f_name, "r")

    count = 0
    for _header_comment, comments, key, value in extract_header_comment_key_value_tuples_from_file(f):
        count += len(key.split())
    return count

if __name__ == "__main__":
    operation = WordCountOperation()
    operation.run_with_standalone_parser()



#!/usr/bin/env python


class Comment(object):
    """ Class representing a comment in the strings file.

    Attributes:
        comment (str): The actual comment content
    """
    def __init__(self, comment):
        """
        Args:
            comment (str): The actual comment content
        """
        self.comment = comment

    def __unicode__(self):
        """ String description of the comment, to be added to the file. """
        return u"%s\n" % self.comment.strip()


class LocalizationEntry(object):
    """ Class representing the localization details for a single string in the strings file.

    Attributes:
        comments (list of str): The comments regarding the localization entry, giving the translation context.
        key (str): The string for translation.
        value (str): The translated value.
    """
    def __init__(self, comments, key, value):
        """
        Args:
            comments (list of Comment objects): The comments regarding the localization entry, giving the translation context.
            key (str): The string for translation.
            value (str): The translated value.
        """
        self.comments = comments
        self.key = key
        self.value = value

    def __unicode__(self):
        """ String description of the entry, to be added to the file. """
        comments_str = u"\n".join([u"/* %s */" % comment for comment in self.comments])
        return u'%s\n"%s" = "%s";\n' % (comments_str, self.key, self.value)

    def add_comments(self, comments):
        """ Add comments to the localization entry

        Args:
            comments (list of str): The comments to be added to the localization entry.
        """
        for comment in comments:
            if comment not in self.comments and len(comment) > 0:
                self.comments.append(comment)
            if len(self.comments[0]) == 0:
                self.comments.pop(0)
